//+------------------------------------------------------------------+
//|                                                         OsMA.mq5 |
//|                                            Copyright 2019, ernst |
//|                             https://www.mql5.com/en/users/pippod |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, ernst"
#property link      "https://www.mql5.com/en/users/pippod"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 6
#property indicator_plots   1
//--- plot OsMA
#property indicator_label1  "OsMA"
#property indicator_type1   DRAW_COLOR_HISTOGRAM
#property indicator_color1  clrRoyalBlue,clrFireBrick,clrYellow
#property indicator_style1  STYLE_SOLID
#property indicator_width1  5
#property indicator_applied_price PRICE_CLOSE
//--- input parameters
input int      FastMAPeriod=12;
input int      SlowMAPeriod=26;
input int      SignalPeriod=9;
input ENUM_MA_METHOD MAMethod=MODE_LWMA;
//--- indicator buffers
double         OsMABuffer[];
double         ColorBuffer[];
double         FastMABuffer[];
double         SlowMABuffer[];
double         MacdBuffer[];
double         SignalBuffer[];

#include <MovingAverages.mqh>
int weightA,weightB,weightC;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   IndicatorSetString(INDICATOR_SHORTNAME,StringFormat("OsMA %s(%d,%d,%d)",StringSubstr(EnumToString(MAMethod),5),FastMAPeriod,SlowMAPeriod,SignalPeriod));
//--- indicator buffers mapping
   SetIndexBuffer(0,OsMABuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ColorBuffer,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(2,FastMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,SlowMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,MacdBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,SignalBuffer,INDICATOR_CALCULATIONS);
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
  {
//---
   switch(MAMethod)
     {
      case MODE_SMA:
         SimpleMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,price,FastMABuffer);
         SimpleMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,price,SlowMABuffer);
         break;
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,price,FastMABuffer);
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,price,SlowMABuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,price,FastMABuffer);
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,price,SlowMABuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,price,FastMABuffer,weightA);
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,price,SlowMABuffer,weightB);
     }
//---
   int limit=!prev_calculated?FastMAPeriod:prev_calculated-1;
   for(int i=limit;i<rates_total && !_StopFlag;i++)
      MacdBuffer[i]=FastMABuffer[i]-SlowMABuffer[i];
//---
   /*switch(MAMethod)
     {
      case MODE_SMA:*/
         SimpleMAOnBuffer(rates_total,prev_calculated,0,SignalPeriod,MacdBuffer,SignalBuffer);
         /*break;
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,SignalPeriod,MacdBuffer,SignalBuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,SignalPeriod,MacdBuffer,SignalBuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,SignalPeriod,MacdBuffer,SignalBuffer,weightC);
     }*/
//---
   limit=!prev_calculated?SlowMAPeriod:prev_calculated-1;
   for(int i=limit;i<rates_total && !_StopFlag;i++)
     { 
      OsMABuffer[i]=MacdBuffer[i]-SignalBuffer[i];
      ColorBuffer[i]=OsMABuffer[i]>OsMABuffer[i-1]?0:OsMABuffer[i]<OsMABuffer[i-1]?1:2;
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
